import pytz


common_timezones = {
    'est': pytz.timezone('US/Eastern'),
    'edt': pytz.timezone('US/Eastern'),
    'eastern': pytz.timezone('US/Eastern'),
    'cst': pytz.timezone('US/Central'),
    'cdt': pytz.timezone('US/Central'),
    'central': pytz.timezone('US/Central'),
    'mst': pytz.timezone('US/Mountain'),
    'mdt': pytz.timezone('US/Mountain'),
    'mountain': pytz.timezone('US/Mountain'),
    'pst': pytz.timezone('US/Pacific'),
    'pdt': pytz.timezone('US/Pacific'),
    'pacific': pytz.timezone('US/Pacific'),
    'utc': pytz.utc,
    'gmt': pytz.utc,
    'universal': pytz.utc,
    'one timezone to rule them all': pytz.utc
}



try:
    from setuptools import setup
except ImportError:
    from distutils import setup


def load_readme():
    readme_file = "README.rst"
    try:
        return open(readme_file, 'r').read()
    except Exception:
        raise RuntimeError("Cannot find readme file {fname}.".format(
            fname=readme_file))


def load_version():
    """Open and parse out the version number from the _version.py module.

    Because of namespacing wtftz.wtfz.converter.convert to wtftz.convert,
    which has the dependencies listed below, I can't import __version__ from
    __init__.py. Open and parse wtftz/_version.py instead.

    Inspired by http://stackoverflow.com/a/7071358
    """
    import re
    version_file = "wtftz/_version.py"
    version_line = open(version_file).read().rstrip()
    vre = re.compile(r'__version__ = "([^"]+)"')
    matches = vre.findall(version_line)
    if matches and len(matches) > 0:
        return matches[0]
    else:
        raise RuntimeError(
            "Cannot find version string in {version_file}.".format(
                version_file=version_file))

version = load_version()
long_description = load_readme()

setup(
    name='wtftz',
    version=version,
    description="Convert a timestamp from one timezone to another",
    long_description=long_description,
    author='Steven Buss',
    author_email='steven.buss@gmail.com',
    url='https://github.com/sbuss/wtftz',
    download_url=('https://github.com/sbuss/wtftz/tarball/v%s' % version),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7",
    ],
    packages=[
        'wtftz',
    ],
    scripts=[
        'bin/wtftz',
    ],
    install_requires=[
        'python-dateutil>=1.5',
        'pytz',
    ],
)



import datetime

from dateutil import parser as date_parser
import pytz

from .timezones import common_timezones


def convert(timestamp, to_tz="utc", from_tz="utc", naive=True):
    """Convert a timestamp from one timezone to another.

    Args:
        timestamp: The timestamp you want to convert.
        to_tz: The timezone you want to end up in.
        from_tz: The timezone of the original timestamp, if needed.
        naive: If True, then strip the tzinfo from the converted timestamp,
               if False then leave it.
    Returns a timestamp in the requested timezone.
    """
    from_timezone = common_tz_name_to_real_tz(from_tz)
    to_timezone = common_tz_name_to_real_tz(to_tz)
    timestamp = parse_timestamp(timestamp)
    if not hasattr(timestamp, 'tzinfo') or timestamp.tzinfo is None:
        timestamp = from_timezone.localize(timestamp)
    timestamp = timestamp.astimezone(to_timezone)
    if naive:
        return timestamp.replace(tzinfo=None)
    else:
        return timestamp


def common_tz_name_to_real_tz(name):
    """Convert the name of a timezone to a real timezone.

    Args:
        name: The name of the timezone. eg "est" or "US/Eastern"
    Returns a tzinfo.
    """
    if isinstance(name, datetime.tzinfo):
        return name
    if name in common_timezones:
        return common_timezones[name]
    try:
        return pytz.timezone(name)
    except Exception:
        pass
    return pytz.UTC


def parse_timestamp(timestamp):
    """Try to parse the given timestamp.

    Args:
        timestamp: The timestamp you want to parse. Accepts epoch,
                   isoformat, and anything python-dateutil can handle.
    Returns a timestamp.
    """
    if isinstance(timestamp, datetime.datetime) or \
            isinstance(timestamp, datetime.time):
        return timestamp

    try:
        timestamp = float(timestamp)
        # Must have an epoch
        try:
            return datetime.datetime.fromtimestamp(timestamp)
        except Exception:
            pass
    except Exception:
        pass

    timestamp = str(timestamp)
    try:
        return date_parser.parse(timestamp)
    except Exception:
        pass

    raise ValueError("Cannot parse timestamp {ts}".format(ts=timestamp))
